#include <stdlib.h>
#include "fireballs.h"

/*****************************************************************************\
| The FireBalls particle                                                      |
\*****************************************************************************/
FireBalls::FireBalls(OFF_Object *newBallLink, float nx, float ny, float nz) : Particle()
{
    origx = nx;
    origy = ny;
    origz = nz;
    ballLink = newBallLink;
    ReInit(origx, origy, origz);
}

FireBalls::~FireBalls()
{
}

void FireBalls::Draw()
{
    glDisable(GL_TEXTURE_2D);
//    glDisable(GL_BLEND);
    glDisable(GL_LIGHTING);
    glPushMatrix();
    glTranslatef(x, y, z);
    glRotatef(ax, 1.0, 0.0, 0.0);
    glRotatef(ay, 0.0, 1.0, 0.0);
    glRotatef(az, 0.0, 0.0, 1.0);
    ballLink->DrawColor();
    glPopMatrix();
    glEnable(GL_LIGHTING);
//    glEnable(GL_BLEND);
    glEnable(GL_TEXTURE_2D);
}

void FireBalls::Calc()
{
    x += dx;
    y += dy;
    z += dz;
    ax += rx;
    ay += ry;
    az += rz;
    dy -= 0.1;
    if (y < -8.0)
        ReInit(origx, origy, origz);
}

void FireBalls::ReInit(float nx, float ny, float nz)
{
    x = nx;
    y = ny;
    z = nz;
    rx = ((10.0*rand()/(RAND_MAX+1.0))-5.0);
    ry = ((10.0*rand()/(RAND_MAX+1.0))-5.0);
    rz = ((10.0*rand()/(RAND_MAX+1.0))-5.0);
    ax = ay = az = 0.0; // no rotation angle
    dy = 0.8+((4.0*rand())/(RAND_MAX+1.0)/10.0);
    dx = ((5.0*rand())/(RAND_MAX+1.0)-2.5)/10.0;
    dz = ((5.0*rand())/(RAND_MAX+1.0)-2.5)/10.0;

    Vector* tmp = ballLink->GetVectorData();
    for (int i=0; i<ballLink->GetNrVerts(); i++)
    {
        float r, g, b;
        r = 0.6+(4*rand()/(RAND_MAX+1.0)/10.0);
        g = 0.2+(rand()/(RAND_MAX+1.0)/10.0);
        b = 0.1+(rand()/(RAND_MAX+1.0)/10.0);
        tmp[i].SetColor(r, g, b, 1.0);
    }
}


/*****************************************************************************\
| The FireBalls engine that controls the particles                            |
\*****************************************************************************/
FireBallsEngine::FireBallsEngine::FireBallsEngine(int newNrParticles, float nx,
                                                  float ny, float nz)
	: ParticleEngine(newNrParticles, nx, ny, nz)
{
    try {
        ball = new OFF_Object(BALL_MODEL);
    }
    catch (...) {
        throw -1;
    }

    for (int i=0; i<NrParticles; i++)
        ps[i] = new FireBalls(ball, x, y, z);
}

FireBallsEngine::~FireBallsEngine()
{
    delete ball;
    for (int i=0; i<NrParticles; i++)
        delete ps[i];
}

void FireBallsEngine::Draw()
{
    for (int i=0; i<NrParticles; i++)
    {
        if (ps[i]->IsAlive())
            ps[i]->Draw();
    }
}

void FireBallsEngine::Calc()
{
    for (int i=0; i<NrParticles; i++)
    {
        if (ps[i]->IsAlive())
            ps[i]->Calc();
    }
}
