#include "OGL_System.h"
#include <string.h>

OGL_System::OGL_System(int *argc, char *argv[],
                       void (*displayf)(), void (*idlef)(),
                       void (*mousef)(int,int,int,int),
                       void (*nkeyf)(unsigned char,int,int),
                       void (*skeyf)(int,int,int),
                       char *win_name, int X, int Y)
{
    start_time = time(NULL);
    end_time = time(NULL);
    ScreenX = X;
    ScreenY = Y;

    /* font */
    disp = NULL;
    FontList = -1;
    FontInfo = NULL;

    glutInit(argc, argv);
    glutInitDisplayMode(GLUT_DOUBLE|GLUT_RGB|GLUT_DEPTH);
    glutInitWindowSize(X, Y);
    glutCreateWindow(win_name);
    glClearColor(0.0, 0.0, 0.0, 1.0);

    /* Setup projection matrix */
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    gluPerspective(45.0, (float)X/(float)Y, 0.1, 100.0);
    glFrustum(-1.0, 1.0, -1.0, 1.0, 0.0, 1.0);
    glOrtho(-1.0, 1.0, -1.0, 1.0, 0.0, 1.0);

    /* Reset model view matrix stack */
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();

    glShadeModel(GL_SMOOTH);    // Select shade model
    glEnable(GL_DEPTH_TEST);    // enable Z buffer
    glDepthFunc(GL_LEQUAL);     // set type z buffer compare
    glEnable(GL_TEXTURE_2D);    // enable textures

    glutDisplayFunc(displayf);
    glutKeyboardFunc(nkeyf);
    glutSpecialFunc(skeyf);
    glutMouseFunc(mousef);
    glutIdleFunc(idlef);
    glutReshapeFunc(ResizeHandler);

    Load_Font(NULL);
    for (int i=1; i<*argc; i++)
    {
        if (!strncmp(argv[i], "-f", 2))
            glutFullScreen();
    }
}

OGL_System::~OGL_System()
{
    if (disp)
    {
        XFreeFont(disp, FontInfo);
        XCloseDisplay(disp);
        glDeleteLists(FontList, 96);
    }
}

void OGL_System::Restart_FPS_Timer()
{
    frames = 0;
    start_time = time(NULL);
}

void OGL_System::Increment_Frame_Counter()
{
    frames++;
    end_time = time(NULL);
}

float OGL_System::Get_FPS()
{
    return (float)frames/(float)(end_time-start_time);
}

void OGL_System::Load_Font(char *fnt)
{
    if (disp)
    {
        XFreeFont(disp, FontInfo);
        XCloseDisplay(disp);
        glDeleteLists(FontList, 96);
    }
    FontList = glGenLists(96);  // 96 chars
    disp = XOpenDisplay(NULL);
    if (fnt)
        FontInfo = XLoadQueryFont(disp, fnt);
    else
        FontInfo = XLoadQueryFont(disp, "-adobe-helvetica-medium-r-normal--18-*-*-*-p-*-iso8859-1");
    if (!FontInfo)
    {
        FontInfo = XLoadQueryFont(disp, "fixed");
        if (!FontInfo)
            return;     // no font available?, fallback texture font?
    }
    glXUseXFont(FontInfo->fid, 32, 96, FontList);
    fr = fg = fb = 1.0;
}

void OGL_System::Set_Font_Color(float r, float g, float b)
{
    fr = r;
    fg = g;
    fb = b;
}

void OGL_System::Print(float x, float y, char *s)
{
    glMatrixMode(GL_PROJECTION);
    glPushMatrix();
    glLoadIdentity();
    glMatrixMode(GL_MODELVIEW);
    glPushMatrix();
    glLoadIdentity();
    glTranslatef(0.0, 0.0, -1.0);
    glColor4f(fr, fg, fb, 1.0);
    glDisable(GL_TEXTURE_2D);
    glDisable(GL_LIGHTING);
    glRasterPos2f(x, y);
    glPushAttrib(GL_LIST_BIT);
    glListBase(FontList - 32);
    glCallLists(strlen(s), GL_UNSIGNED_BYTE, s);
    glEnable(GL_LIGHTING);
    glEnable(GL_TEXTURE_2D);
    glPopAttrib();
    glPopMatrix();
    glMatrixMode(GL_PROJECTION);
    glPopMatrix();
    glMatrixMode(GL_MODELVIEW);
}

void ResizeHandler(int new_width, int new_height)
{
    glViewport(0, 0, new_width, new_height);	// Reset The Current Viewport And Perspective Transformation
    glMatrixMode(GL_PROJECTION);        // Load projection matrix
    glLoadIdentity();
    gluPerspective(45.0f, (float)new_width/(float)new_height, 0.1f, 100.0f);
    glMatrixMode(GL_MODELVIEW);         // Load model matrix
}
