#ifndef __LINALG_H
#define __LINALG_H

/*
 This lib defines linear algebra functions and objects for 3D space

 (C) John v/d Kamp, blade_@dds.nl
 License: GPL
 */

class Vector;

class Vertex {
private:
protected:
public:
    float x, y, z;
    float r, g, b, a;

    Vertex();
    Vertex(Vertex &v);
    Vertex(Vertex *v);
    Vertex(float newX, float newY, float newZ);
    virtual ~Vertex();

    void GetPoint(float *X, float *Y, float *Z);
    void SetPoint(float newX, float newY, float newZ);
    void SetPoint(Vertex &v);

    void GetColor(float *R, float *G, float *B, float *A);
    void SetColor(float R, float G, float B, float A);

    friend class Vector;
};

class Vector : public Vertex {
private:
protected:
public:
    Vector();
    Vector(Vertex &v);
    Vector(Vertex *v);
    Vector(float newX, float newY, float newZ);
    ~Vector();

    float Length();
    void Normalize();
    Vector& operator+(const Vector &b);
    void operator+=(const Vector &b);
    Vector& operator-(const Vector &b);
    void operator-=(const Vector &b);
    void Scale(Vector &v);
    void Scale(float f);

    friend float DotProduct(Vector &a, Vector &b);
    friend Vector& CrossProduct(Vector &a, Vector &b);
    friend float Angle(Vector &a, Vector &b);
};

float DotProduct(Vector &a, Vector &b);
Vector& CrossProduct(Vector &a, Vector &b);
float Angle(Vector &a, Vector &b);

#endif
