#include <fstream.h>
#include "off_object.h"

OFF_Object::OFF_Object(char *name)
{
    ifstream f;
    char off[4];
    int foo, i;

    f.open(name);
    if (!f)
        throw -1;
    f >> off;
    if (strcmp(off,"OFF"))
    {
        f.close();
        throw -2;
    }
    f >> nverts >> npolys >> foo;

    data = new Vector[nverts];
    for (i=0; i<nverts; i++)
    {
        float x, y, z;
        f >> x >> y >> z;
        data[i].SetPoint(x, y, z);
        data[i].SetColor(1.0, 1.0, 1.0, 1.0);
    }
    quads = new quad_struct[npolys];
    for (i=0; i<npolys; i++)
    {
        f >> foo >> quads[i].a >> quads[i].b >> quads[i].c >> quads[i].d;
        quads[i].normal =
            new Vector(CrossProduct(data[quads[i].b] - data[quads[i].a],
                                    data[quads[i].c] - data[quads[i].a]));
        quads[i].normal->Normalize();
    }
    f.close();
}

OFF_Object::~OFF_Object()
{
    delete [] data;
    delete [] quads;
}

void OFF_Object::Draw()
{
    glBegin(GL_QUADS);
    for (int i=0; i<npolys; i++)
    {
        float x, y, z;
        quads[i].normal->GetPoint(&x, &y, &z);
        glNormal3f(x, y, z);
        glVertex3f(data[quads[i].a].x, data[quads[i].a].y, data[quads[i].a].z);
        glVertex3f(data[quads[i].b].x, data[quads[i].b].y, data[quads[i].b].z);
        glVertex3f(data[quads[i].c].x, data[quads[i].c].y, data[quads[i].c].z);
        glVertex3f(data[quads[i].d].x, data[quads[i].d].y, data[quads[i].d].z);
    }
    glEnd();
/*
    k++;
    if (k>npolys)
        k = npolys;
*/
}

void OFF_Object::DrawColor()
{
    glBegin(GL_QUADS);
    for (int i=0; i<npolys; i++)
    {
        float x, y, z;
        float r, g, b, a;
        quads[i].normal->GetPoint(&x, &y, &z);
        glNormal3f(x, y, z);
        data[quads[i].a].GetColor(&r, &g, &b, &a);
        glColor4f(r, g, b, a);
        glVertex3f(data[quads[i].a].x, data[quads[i].a].y, data[quads[i].a].z);
        data[quads[i].a].GetColor(&r, &g, &b, &a);
        glColor4f(r, g, b, a);
        glVertex3f(data[quads[i].b].x, data[quads[i].b].y, data[quads[i].b].z);
        data[quads[i].a].GetColor(&r, &g, &b, &a);
        glColor4f(r, g, b, a);
        glVertex3f(data[quads[i].c].x, data[quads[i].c].y, data[quads[i].c].z);
        data[quads[i].a].GetColor(&r, &g, &b, &a);
        glColor4f(r, g, b, a);
        glVertex3f(data[quads[i].d].x, data[quads[i].d].y, data[quads[i].d].z);
    }
    glEnd();
/*
    k++;
    if (k>npolys)
        k = npolys;
*/
}

int OFF_Object::GetNrVerts()
{
    return nverts;
}

int OFF_Object::GetNrPolys()
{
    return npolys;
}

Vector* OFF_Object::GetVectorData()
{
    return data;
}

quad_struct* OFF_Object::GetQuadData()
{
    return quads;
}
