#include <string.h>
#include "particle.h"

/*****************************************************************************\
| The Particle itself                                                         |
\*****************************************************************************/
Particle::Particle()
{
    forceLink = NULL;
}

Particle::~Particle()
{
}

int Particle::IsAlive()
{
    return alive;
}

void Particle::SetForce(Physics *newForceLink)
{
    forceLink = newForceLink;
}


/*****************************************************************************\
| The Particle engine that controls the particles                             |
\*****************************************************************************/
ParticleEngine::ParticleEngine(int newNrParticles, float nx, float ny, float nz)
{
    NrParticles = newNrParticles;
    ps = new (Particle*)[NrParticles];
    x = nx;
    y = ny;
    z = nz;
    force = NULL;
}

ParticleEngine::~ParticleEngine()
{
    delete [] ps;
}

void ParticleEngine::SetNewForce(Physics *newForce)
{
    force = newForce;
    for (int i=0; i<NrParticles; i++)
        ps[i]->SetForce(force);
}

/*****************************************************************************\
| The Particle engine manager that controls the particle engines              |
\*****************************************************************************/
ParticleManager::ParticleManager(int newMaxParticleEngines)
{
    MaxParticleEngines = newMaxParticleEngines;
    pes = new (ParticleEngine*)[MaxParticleEngines];
    for (int i=0; i<MaxParticleEngines; i++)
        pes[i] = NULL;
}

ParticleManager::~ParticleManager()
{
    delete [] pes;
}

void ParticleManager::DrawAll()
{
    for (int i=0; i<MaxParticleEngines; i++)
        if (pes[i])
            pes[i]->Draw();
}

void ParticleManager::Draw(int index)
{
    if (pes[index])
        pes[index]->Draw();
}

void ParticleManager::CalcAll()
{
    for (int i=0; i<MaxParticleEngines; i++)
        if (pes[i])
            pes[i]->Calc();
}

void ParticleManager::Calc(int index)
{
    if (pes[index])
        pes[index]->Calc();
}

void ParticleManager::AddParticleEngine(int index, ParticleEngine *pe)
{
    if ((!pe) || (index<0) || (index>=MaxParticleEngines))
        return;
    pes[index] = pe;
}

void ParticleManager::RemoveParticleEngine(int index)
{
    if ((index<0) || (index>=MaxParticleEngines))
        return;
    pes[index] = NULL;
}
