#ifndef __PARTICLE_H
#define __PARTICLE_H

#include "physics.h"

/* Base class: just to inherit from */
class Particle {
protected:
    int alive;
    float x, y, z;	// coords of the particle
    float dx, dy, dz;	// delta values for direction changes
    float rx, ry, rz;	// rotation speeds
    float ax, ay, az;   // angle values
    float r, g, b, a;	// color of the particle and alpha value
    Physics *forceLink; // link to ParticleEngine::force
public:
    Particle();
    virtual ~Particle();

    virtual void Draw() = 0;
    virtual void Calc() = 0;
    int IsAlive();
    void SetForce(Physics *newForceLink);
};

class ParticleEngine {
protected:
    Particle **ps;
    int NrParticles;
    float x, y, z;      // start point
    Physics *force;
public:
    ParticleEngine(int newNrParticles, float nx, float ny, float nz);
    virtual ~ParticleEngine();

    virtual void Draw() = 0;
    virtual void Calc() = 0;
    void SetNewForce(Physics *newForce);
};

class ParticleManager {
private:
    int MaxParticleEngines;
    ParticleEngine **pes;
public:
    ParticleManager(int newMaxParticleEngines);
    ~ParticleManager();

    void DrawAll();
    void Draw(int index);
    void CalcAll();
    void Calc(int index);
    void AddParticleEngine(int index, ParticleEngine *pe);
    void RemoveParticleEngine(int index);
};

#endif
