#include <string.h>
#include "texture.h"

Texture::Texture()
{
    disp = XOpenDisplay(NULL);
    id = Imlib_init(disp);
    im = NULL;
    x_size = y_size = 0;
    data = NULL;
    masked = 0;
    loaded = 0;
}

Texture::~Texture()
{
    if (loaded)
        glDeleteTextures(1,&tnr[0]);
    if (masked)
        glDeleteTextures(1,&tnr[1]);
    if (data)
        delete [] data;
    XCloseDisplay(disp);
}

int Texture::LoadAndConvert(char *fpic, int trans, int flipxy)
{
    if (loaded)
        return 0;                       // only one texture per object/class

    im = Imlib_load_image(id, fpic);
    if (!im)
        return 0;
    if (flipxy&FLIP_X)
        Imlib_flip_image_horizontal(id, im);
    if (flipxy&FLIP_Y)
        Imlib_flip_image_vertical(id, im);
    x_size = im->rgb_width;
    y_size = im->rgb_height;

    if (im->alpha_data)
    {
        data = new (unsigned char)[x_size*y_size*4]; // RGBA
        if (!data)
            return 0;
        /* copy alpha data to one array: rgb & a -> rgba */
        for (int a=0; a<x_size*y_size; a++)
        {
            data[a*4+0] = im->rgb_data[a*3+0];
            data[a*4+1] = im->rgb_data[a*3+1];
            data[a*4+2] = im->rgb_data[a*3+2];
            data[a*4+3] = im->alpha_data[a];
        }
    } else if (trans==NO_ALPHA)
    {
        data = new (unsigned char)[x_size*y_size*3]; // RGB
        if (!data)
            return 0;
        memcpy(data,im->rgb_data,x_size*y_size*3);
    }
    else
    {
        data = new (unsigned char)[x_size*y_size*4]; // RGBA
        if (!data)
            return 0;
        /* convert rgb -> rgba */
        for (int a=0; a<x_size*y_size; a++)
        {
            data[a*4+0] = im->rgb_data[a*3+0];
            data[a*4+1] = im->rgb_data[a*3+1];
            data[a*4+2] = im->rgb_data[a*3+2];
            data[a*4+3] = (unsigned char)rint((im->rgb_data[a*3+0]+im->rgb_data[a*3+1]+im->rgb_data[a*3+2])/3.0);
//            data[a*4+3] = ((im->rgb_data[a*3+0]+im->rgb_data[a*3+1]+im->rgb_data[a*3+2])==0)?0:1;
        }
    }
    return 1;
}

/*
 * Load texture without mask layer
 */
int Texture::Load(char *fpic, int trans, int flipxy)
{
    int rc;
    if (!(rc = LoadAndConvert(fpic, trans, flipxy)))
        return rc;

    /* Load image into OpenGL driver (video memory) */
    glGenTextures(1, tnr);
    glBindTexture(GL_TEXTURE_2D, tnr[0]);
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_NEAREST);
    if (trans==NO_ALPHA)
    {
        glTexImage2D(GL_TEXTURE_2D, 0, 3, x_size, y_size, 0, GL_RGB, GL_UNSIGNED_BYTE, data);
        gluBuild2DMipmaps(GL_TEXTURE_2D, 3, x_size, y_size, GL_RGB, GL_UNSIGNED_BYTE, data);
    }
    else
    {
        glTexImage2D(GL_TEXTURE_2D, 0, 4, x_size, y_size, 0, GL_RGBA, GL_UNSIGNED_BYTE, data);
        gluBuild2DMipmaps(GL_TEXTURE_2D, 4, x_size, y_size, GL_RGBA, GL_UNSIGNED_BYTE, data);
    }

    Imlib_destroy_image(id, im);
    delete [] data;
    data = NULL;
    masked = 0;
    loaded = 1;
    return 1;
}

/*
 * Load texture with matching mask layer
 */
int Texture::Load(char *fpic, char *fmask, int trans, int flipxy)
{
    int rc;
    if (!(rc = LoadAndConvert(fpic, trans, flipxy)))
        return rc;

    /* Load image into OpenGL driver (video memory) */
    /* first image, then mask */
    glGenTextures(2, tnr);
    glBindTexture(GL_TEXTURE_2D, tnr[0]);
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_NEAREST);
    if (trans==NO_ALPHA)
    {
        glTexImage2D(GL_TEXTURE_2D, 0, 3, x_size, y_size, 0, GL_RGB, GL_UNSIGNED_BYTE, data);
        gluBuild2DMipmaps(GL_TEXTURE_2D, 3, x_size, y_size, GL_RGB, GL_UNSIGNED_BYTE, data);
    }
    else
    {
        glTexImage2D(GL_TEXTURE_2D, 0, 4, x_size, y_size, 0, GL_RGBA, GL_UNSIGNED_BYTE, data);
        gluBuild2DMipmaps(GL_TEXTURE_2D, 4, x_size, y_size, GL_RGBA, GL_UNSIGNED_BYTE, data);
    }
    
    Imlib_destroy_image(id, im);
    im = Imlib_load_image(id, fmask);
    if (!im)
        return 0;
    if (flipxy&FLIP_X)
        Imlib_flip_image_horizontal(id, im);
    if (flipxy&FLIP_Y)
        Imlib_flip_image_vertical(id, im);
    glBindTexture(GL_TEXTURE_2D, tnr[1]);
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_NEAREST);
    glTexImage2D(GL_TEXTURE_2D, 0, 3, im->rgb_width, im->rgb_height, 0, GL_RGB, GL_UNSIGNED_BYTE, im->rgb_data);
    gluBuild2DMipmaps(GL_TEXTURE_2D, 3, im->rgb_width, im->rgb_height, GL_RGB, GL_UNSIGNED_BYTE, im->rgb_data);

    Imlib_destroy_image(id, im);        // kill mask imlib image
    delete [] data;
    data = NULL;
    masked = loaded = 1;
    return 1;
}

int Texture::Load(unsigned char *new_data, int alpha, int width, int height)
{
    if ((!new_data) || (loaded))
        return 0;

    switch (alpha) {
    case NO_ALPHA:              // picture only has RGB elements
        data = new (unsigned char)[width*height*3];
        if (!data)
            return 0;
        memcpy(data,new_data,width*height*3);
        break;
    case USE_ALPHA:             // picture allready has RGBA elements
        data = new (unsigned char)[width*height*4];
        if (!data)
            return 0;
        memcpy(data,new_data,width*height*4);
        break;
    case MAKE_ALPHA:        	// picture only has RGB but wants to be RGBA
        data = new (unsigned char)[width*height*4];
        if (!data)
            return 0;
        /* convert rgb -> rgba */
        for (int a=0; a<width*height; a++)
        {
            data[a*4+0] = new_data[a*3+0];
            data[a*4+1] = new_data[a*3+1];
            data[a*4+2] = new_data[a*3+2];
            data[a*4+3] = (unsigned char)rint((new_data[a*3+0]+new_data[a*3+1]+new_data[a*3+2])/3.0);
        }
        break;
    };

    /* Load image into OpenGL driver (video memory) */
    glGenTextures(1, tnr);
    glBindTexture(GL_TEXTURE_2D, tnr[0]);
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_NEAREST);
    if (alpha==NO_ALPHA)
    {
        glTexImage2D(GL_TEXTURE_2D, 0, 3, width, height, 0, GL_RGB, GL_UNSIGNED_BYTE, data);
        gluBuild2DMipmaps(GL_TEXTURE_2D, 3, width, height, GL_RGB, GL_UNSIGNED_BYTE, data);
    }
    else
    {
        glTexImage2D(GL_TEXTURE_2D, 0, 4, width, height, 0, GL_RGBA, GL_UNSIGNED_BYTE, data);
        gluBuild2DMipmaps(GL_TEXTURE_2D, 4, width, height, GL_RGBA, GL_UNSIGNED_BYTE, data);
    }

    delete [] data;
    data = NULL;
    masked = 0;
    loaded = 1;
    return 1;
}

int Texture::GetID()
{
    return tnr[0];
}

void Texture::Select()
{
    glBindTexture(GL_TEXTURE_2D, tnr[0]);
}

void Texture::SelectMask()
{
    if (masked)
        glBindTexture(GL_TEXTURE_2D, tnr[1]);
    else
        glBindTexture(GL_TEXTURE_2D, tnr[0]);
}
