#include <iostream.h>
#include <stdio.h>      // sprintf
#include <math.h>
#include <time.h>
#include <stdlib.h>
#include <OGL_System.h>
#include <particle.h>
#include <physics.h>
#include <texture.h>
#include "cloud.h"
#include "island.h"
#include "sea.h"
#include "smoke.h"
#include "fireballs.h"

OGL_System *OpenGLSystem;
ParticleManager *pm;
ParticleEngine *cloud1, *cloud2;
ParticleEngine *smoke;
ParticleEngine *fireballs;

Physics *Wind;
Island *island;
Sea *sea;
Texture *sun;

GLfloat Light0Ambient[] = {1.0, 1.0, 0.7, 1.0};  // all around the world
GLfloat Light0Diffuse[] = {1.0, 1.0, 0.9, 1.0};  // all around the world
GLfloat Light0Position[] = {-8.0, 5.0, -12.0, 1.0};
float s, zoom, r;
int paused, scene_rot;

void display()
{
    glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);

    glPushMatrix();
    glTranslatef(0.0, 0.0, zoom);	// zoom in/out
    if (scene_rot)
        glRotatef(s, 1.0, 1.0, 0.0);

    /* Draw lame-ass sun */
    sun->Select();
    glPushMatrix();
    glColor4f(1.0, 1.0, 1.0, 1.0);
    glTranslatef(-5.0, 5.0, -4.0);
    glRotatef(r, 0.0, 0.0, 1.0);
    glBegin(GL_QUADS);
    glTexCoord2f(0.0, 0.0);	glVertex3f(-3.0, -3.0, 0.0);
    glTexCoord2f(1.0, 0.0);	glVertex3f( 3.0, -3.0, 0.0);
    glTexCoord2f(1.0, 1.0);	glVertex3f( 3.0,  3.0, 0.0);
    glTexCoord2f(0.0, 1.0);	glVertex3f(-3.0,  3.0, 0.0);
    glEnd();
    glPopMatrix();

    sea->Draw();
    island->Draw();
    pm->DrawAll();

    /* draw axis */
    /*
    glDisable(GL_DEPTH_TEST);
    glDisable(GL_TEXTURE_2D);
    glColor4f(0.0, 1.0, 0.0, 1.0);
    glBegin(GL_LINES);
    glVertex3f(-10.0, 0.0, 0.0);
    glVertex3f(10.0, 0.0, 0.0);
    glVertex3f(0.0, -10.0, 0.0);
    glVertex3f(0.0, 10.0, 0.0);
    glEnd();
    glEnable(GL_TEXTURE_2D);
    glEnable(GL_DEPTH_TEST);
    glColor4f(1.0, 1.0, 1.0, 1.0);
    */

    glPopMatrix();

    OpenGLSystem->Increment_Frame_Counter();
    char s[16];
    sprintf(s, "FPS: %5.3f s", OpenGLSystem->Get_FPS());
    OpenGLSystem->Print(-0.98, 0.92, (char*)s);
    if (paused)
        OpenGLSystem->Print(-0.15, 0.0, "* Paused *");
    glutSwapBuffers();
}

void scenario()
{
    if (!paused)
    {
        pm->CalcAll();
        sea->Calc();
        r += 0.5;
        s += 0.5;
    }
    glutPostRedisplay();
}

void mouse(int button, int state, int x, int y)
{
    cout << "Mouse pressed:\tbutton" << button << "\tstate " << state << "\tx " << x << "\ty " << y << endl;
}

void key(unsigned char key, int x, int y)
{
    switch (key) {
    case 27:
        {
            cout << "fps: " << OpenGLSystem->Get_FPS() << endl;
            delete OpenGLSystem;
            pm->RemoveParticleEngine(0);
            pm->RemoveParticleEngine(1);
            delete cloud1;
            delete cloud2;
            delete island;
            delete sea;
            delete pm;
            exit(0);
        }
    case 'p':
    case 'P':
        paused = !paused;
        break;
    case 'r':
    case 'R':
        s = 0.0;
        scene_rot = !scene_rot;
        break;
    case '+':
        zoom -= 1.0;
        break;
    case '-':
        zoom += 1.0;
        break;
    default:
        cout << "Dont know " << key << endl;
        break;
    };
}

void specialkeys(int key, int x, int y)
{
}

int main(int argc, char *argv[])
{
    srand(time(NULL));

    /* Allways create the OpenGL System first before using textures! */
    OpenGLSystem = new OGL_System(&argc,argv, display, scenario, mouse, key,
                                  specialkeys, "Smoke Particle Engine test");
    glEnable(GL_BLEND);
    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
    glClearColor(0.1, 0.3, 0.9, 1.0);

    glLightfv(GL_LIGHT0, GL_AMBIENT, Light0Ambient);  // add lighting. (ambient)
    glLightfv(GL_LIGHT0, GL_DIFFUSE, Light0Diffuse);  // add lighting. (diffuse).
/*
    s = 0.0;
    Light0Position[0] = sin(s);
    Light0Position[1] = cos(s);
*/
    glLightfv(GL_LIGHT0, GL_POSITION, Light0Position); // set light position.
    glEnable(GL_LIGHTING);
    glEnable(GL_LIGHT0);
    
    sun = new Texture;
    sun->Load("data/sun.png", MAKE_ALPHA, FLIP_Y);

    cout << "Creating Sea and Island: "; fflush(stdout);
    try {
        island = new Island;
    }
    catch(...) {
        cout << "Error on creating Island. Bailing out..." << endl;
        return -1;
    }
    sea = new Sea;
    cout << "Done." << endl;

    /* Setup particle engines (2x cloud, 1x smoke, 1x fireballs */
    cout << "Creating Particle Engines: "; fflush(stdout);
    Wind = new Physics(0.01, 0.005, 0.01);
    cloud1 = new CloudEngine(20, 6.0, 2.0, -5.0, 3.0, -2.0);
//    cloud1->SetNewForce(Wind);
    cloud2 = new CloudEngine(30, 12.0, 2.0, 5.0, 3.5, -2.0);
//    cloud2->SetNewForce(Wind);
    smoke = new SmokeEngine(20, 0.0, 0.0, 0.0);
    smoke->SetNewForce(Wind);
    try {
        fireballs = new FireBallsEngine(15, 0.0, -1.5, 0.0);
    }
    catch (...) {
        cout << "Error on creating particle engines. Bailing out..." << endl;
        return -1;
    }
    fireballs->SetNewForce(Wind);
    pm = new ParticleManager(4);
    pm->AddParticleEngine(0, fireballs);
    pm->AddParticleEngine(1, cloud1);
    pm->AddParticleEngine(2, cloud2);
    pm->AddParticleEngine(3, smoke);
    cout << "Done." << endl;

    /* Run! */
    zoom = -15.0;
    scene_rot = paused = 0;
    OpenGLSystem->Restart_FPS_Timer();
    glutMainLoop();
    return 0;   // never reached
}
